clear all
close all

beat_note_order = 5;

%% Parameters
defect_case = 1; %choose 1 or 2, it changes the boundary conditions for the reflection and transmission coefficients
                 % case 1: no phase shift; case 2: pi-phase shift at the defect;
R_defect = 1; %intensity reflectivity at the defect, all other coefficients will be automatically calculated
L = 1; %length of the resonator, i.e. perimeter of the ring

Am1 = 1; %amplitude of the field component circulating in the CCW direction for mode 1
m1 = 10; %index of resonator mode 1

Am2 = 1; %amplitude of the field component circulating in the CCW direction for mode 2
m2 = m1+beat_note_order; %index of resonator mode 2

c = 1; %arbitrary speed of light

x = linspace(0,L,1000); %spatial coordinate

plot_style = '3D'; %choose between 'linear' and '3D' plots
show_animation = 0; %choose 1 to show animation, else 0
cavity = 'ring'; %choose between 'ring' and 'FP'; due to the equations that were calculated for a ring,
               %in the FP case the spacing between the modes can only be a multiple of 2FSR
wave = 'sine'; %assuming 'sine' or 'cosine' like waves (node or antinode of optical fields at the defect), beat results should not change
gsol_index = 1; %choose index (1 or 2) of solution of the trascendental equation to get the gain coefficient

%% Defect boundary conditions
theta = acos(sqrt(R_defect));
rm = cos(theta); %reflectivity for the field on the left of the defect
tm = sin(theta); %trasmissivity for the field on the left of the defect

switch defect_case
    case 1 %zero phase-shift
        rp = cos(theta); %reflectivity for the field on the right of the defect
        tp = -sin(theta); %trasmissivity for the field on the right of the defect
    case 2 %pi phase-shift
        rp = -cos(theta); %reflectivity for the field on the right of the defect
        tp = sin(theta); %trasmissivity for the field on the right of the defect
end

switch cavity
    case 'ring'
    case 'FP'
        tp = 0;
        tm = 0;
end

%% Wavevector and angular frequency of the modes
k1 = 2*pi/L*m1;
w1 = c*k1;

k2 = 2*pi/L*m2;
w2 = c*k2;

%% Define time vector
Tbeat = L/(abs(m1-m2)*c); %beat period

Np = 1000; %number of points in a beat period: needs to be sufficiently large or artifacts may arise

if show_animation
    t = 0:Tbeat/Np:10*Tbeat; %time vector defined to sample sufficiently the time period
else
    t = 0:Tbeat/Np:1*Tbeat; %time vector defined to sample sufficiently the time period
end


%% Solve trascendental equation to obtain gain coefficient
syms gx %unknown
geqn = 1-exp(gx*L/2)*tp == exp(gx*L/2)*rp/(1-exp(gx*L/2)*tm)*exp(gx*L/2)*rm ;
gsols = solve(geqn,gx); %solutions

if length(gsols) < 1
    disp('No solutions were found to the trascendental equation. Check')
else
    g = eval(gsols(gsol_index)); %net gain coefficient solution
end

%% Compute standing waves of the modes
switch wave
    case 'cosine'
        wavesign = +1;
    case 'sine'
        wavesign = -1;
end

Apcoeff = wavesign*exp(g*L/2)*rp/(1-exp(g*L/2)*tm); %coefficient that multiply Am to give Ap, it's the same for all modes

Ap1 = Am1*Apcoeff; %amplitude of the field component circulating in the CW direction for mode 1
Ap2 = Am2*Apcoeff; %amplitude of the field component circulating in the CW direction for mode 2

I = zeros(1,length(x));
Itot = zeros(length(t),length(x)); % vector to save maxima at each position in the cavity during run
angle = x/L*360; %ring coordinate angle

figure
fnum = get(gcf,'Number');
switch plot_style
    case 'linear'
        plothandle = plot(angle,I,'LineWidth',1);
        plothandle.YDataSource = 'I';
        xlabel('Angle ({\circ})')
        ylabel('Intensity (a.u.)')
        set(gcf,'Position',[156   444   461   193])
        xlim([0 360])
        vertlim_max = 1;
    case '3D'
        R = 1; %radius for the circle plot
        plothandle = plot3(R*cosd(angle),R*sind(angle),I,'LineWidth',1);
        plothandle.ZDataSource = 'I';
        axis square
        box on
        zlabel('Intensity (a.u.)')
        view([-41, 33])
        vertlim_max = 1;
end

for i=1:length(t)
    Ep1 = Ap1*exp(g*x/2).*exp(1i*(k1*x - w1*t(i))); %CW component of mode 1
    Em1 = Am1*exp(g*(L-x)/2).*exp(1i*(k1*x + w1*t(i))); %CCW component of mode 1
    
    Ep2 = Ap2*exp(g*x/2).*exp(1i*(k2*x - w2*t(i))); %CW component of mode 2
    Em2 = Am2*exp(g*(L-x)/2).*exp(1i*(k2*x + w2*t(i))); %CCW component of mode 2
    
    Etot = Ep1 + Em1 + Ep2 + Em2;
    %I = (abs(Etot + conj(Etot))).^2;
    I = (abs(Etot)).^2;
    %I = (Ep1 + Em1 + conj(Ep1 + Em1)).*(Ep2 + Em2 + conj(Ep2 + Em2)); %beat only
    
    Itot(i,:) = I;
    
    if max(I) > vertlim_max
        vertlim_max = max(I);
        switch plot_style
            case 'linear'
                ylim([0 vertlim_max])
            case '3D'
                zlim([0 vertlim_max])
        end
    end
    

    if show_animation
        figure(fnum)
        refreshdata(plothandle);
        pause(0.02)
    end
    
end

close(figure(fnum))


%% Plot beat power pattern

P = (max(Itot)-min(max(Itot))).^2; %beat power --> DIRTY WAY
Pnorm = P/max(P);
Psat = max(Pnorm)*2;
n = 1./(1+Pnorm/Psat);
nnorm = n/max(n);

figure
set(gcf, 'Position', [617   435   215   126])
subplot(2,1,1)
plot(angle,Pnorm,'LineWidth',1,'Color',[0.5019 0.5019 0.5019]);
pbaspect([1.0000    0.2534    0.2534])
%xlabel('Angle ({\circ})')
%ylabel('Power (a.u.)')
box off
xlim([0,360])
ylim([0 1])
set(gca,'xtick',[],'xticklabel',[],'ytick',[],'yticklabel',[])
subplot(2,1,2)
plot(angle,nnorm,'LineWidth',1,'Color',[0.5019 0.5019 0.5019]);
pbaspect([1.0000    0.2534    0.2534])
%xlabel('Angle ({\circ})')
%ylabel('Power (a.u.)')
box off
xlim([0,360])
ylim([0 1])
set(gca,'xtick',[],'xticklabel',[],'ytick',[],'yticklabel',[])

figure
set(gcf, 'Position', [950   339   213   353])
R = 1; %radius for the circle plot
subplot(2,1,1)
hold all
plot3(R*cosd(angle),R*sind(angle),Pnorm,'LineWidth',1,'Color',[0.5019 0.5019 0.5019]);
plot3(R*cosd(angle),R*sind(angle),zeros(1,length(Pnorm)),'-k','LineWidth',0.5,'Color',[0.24705882370472 0.24705882370472 0.24705882370472]);
axis square
box off
%zlabel('Intensity (a.u.)')
view([-41, 33])
vertlim_max = 1;
zlim([0 1])
axis off
pbaspect([1 1 0.3])
subplot(2,1,2)
hold all
plot3(R*cosd(angle),R*sind(angle),nnorm,'LineWidth',1,'Color',[0.5019 0.5019 0.5019]);
plot3(R*cosd(angle),R*sind(angle),zeros(1,length(nnorm)),'-k','LineWidth',0.5,'Color',[0.24705882370472 0.24705882370472 0.24705882370472]);
axis square
box off
%zlabel('Intensity (a.u.)')
view([-41, 33])
vertlim_max = 1;
zlim([0 1])
axis off
pbaspect([1 1 0.3])


